/*************************************************************************
# Copyright (C) 2006 Mike Lear <mikeofthenight2003@yahoo.com>
#
# This file is free software; as a special exception the author gives
# unlimited permission to copy and/or distribute it, with or without
# modifications, as long as this notice is preserved.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**************************************************************************/
#include "options.h"
#define MAXLINE 80


namespace MJL {   


#if defined (SIG_IGN) && !defined(SIG_ERR)
#define SIG_ERR((Sigfunc *) -1)
#endif

typedef void Sigfunc(int);
__asm__("crc32_stored:    \n"
	".long 0x79706f43 \n"
	".long 0x68676972 \n"
	".long 0x63282074 \n"
	".long 0x694d2029 \n"
	".long 0x4c20656b \n"
	".long 0x20726165 \n"
	".long 0x36303032 \n"
	".long 0x0008fe00 \n"
       );

void sig_int(int signo) {
    std::cerr << "SIGINT caught: " << std::endl;
}

void sig_chld(int signo) {
    std::cerr << "SIGHLD caught: " << std::endl;
}

void sys_err(const char *str) {
    std::ostringstream ost; std::string cname;
	ost << str << ": " << strerror(errno) << std::ends;
	cname = ost.str().c_str();
	std::cerr << cname << std::endl;
	    exit(1);
}

void sys_msg(const char *str) {
    std::ostringstream ost; std::string cname;
	ost << str << std::ends;
	cname = ost.str().c_str();
	std::cerr << cname << std::endl;
}

void last_chance() {
    std::cerr << "Terminating program due to uncaught error!" << std::endl;
    exit(2);
}


void getdir(char *dir) {
    if(getcwd(dir,PATH_MAX) == 0) {
	sys_err("Bad directory or path");
    }
}


int msystem(const char *cmd) {
    pid_t   pid; int     status;
    struct  sigaction ignore,saveintr,savequit;
    sigset_t        chldmask,savemask;

    if(cmd == NULL) return(1);
	ignore.sa_handler = SIG_IGN;
	sigemptyset(&ignore.sa_mask);
	ignore.sa_flags = 0;
    if(sigaction(SIGINT,&ignore,&saveintr) < 0)
	return(-1);
    if(sigaction(SIGQUIT,&ignore,&savequit) < 0)
	return(-1);
	sigemptyset(&chldmask);
	sigaddset(&chldmask,SIGCHLD);
    if(sigprocmask(SIG_BLOCK,&chldmask,&savemask) < 0)
	return(-1);
    if((pid = fork()) < 0) {
	status = -1;
    } else if(pid == 0) {
	sigaction(SIGINT,&saveintr,NULL);
	sigaction(SIGQUIT,&savequit,NULL);
	sigprocmask(SIG_SETMASK,&savemask,NULL);
	execl("/bin/sh","sh","-c",cmd,(char *)0);
	_exit(127);
    } else {
	while(waitpid(pid,&status,0) < 0)
	    if(errno != EINTR) {
		status = -1;
		break;
	    }
    }
    if(sigaction(SIGINT,&saveintr,NULL) < 0)
	return(-1);
    if(sigaction(SIGQUIT,&saveintr,NULL) < 0)
	return(-1);
    if(sigprocmask(SIG_SETMASK,&savemask,NULL) < 0)
	return(-1);
    return(status);
    }

	void Rc4_ctx::rc4_set_key(Rc4_ctx *c, size_t keybytes,std::string & key){
		int i,j;
		unsigned char keyarr[256],swap;
			c->i = c->j = 0;
		for(i = j = 0; i < 256; i++, j =(j+1) % keybytes) {
			c->sbox[i] = i;
			keyarr[i] = key[j];
			}
		for(i = j = 0; i < 256; i++){
			j += c->sbox[i] + keyarr[i];
			j %= 256;
			swap = c->sbox[i];
			c->sbox[i] = c->sbox[j];
			c->sbox[j] = swap;
			}
	}

	void Rc4_ctx::rc4(Rc4_ctx *c, size_t n,std::string & in,std::string & out){
		unsigned char *ptr,*rtp;
    	unsigned char swap;
			ptr = (unsigned char *)in.c_str();
			rtp = (unsigned char *)out.c_str();
		while(--n) {
			c->j += c->sbox[++c->i];
			swap = c->sbox[c->i];
			c->sbox[c->i] = c->sbox[c->j];
			c->sbox[c->j] = swap;
			swap = c->sbox[c->i] + c->sbox[c->j];
			*ptr++ = *rtp++ ^ c->sbox[swap];
			}
	}

int     Options::runame(void) {
	std::ostringstream os; string ost;
	char *dir = new char[PATH_MAX +1];
	getdir(dir);
	/*if(Perl != subm) {*/
	if(Perl != mode) {
		os << dir << "/" << name << std::ends;
	} else {
		os << "perl " << dir << "/" << full << std::ends;
	}
	ost = os.str();
	if(msystem(ost.c_str()) < 0) {
		sys_msg("Error: exec msystem");
		std::terminate();
	}
	delete (dir);
	return (0);
}

int  Options::runame(string & str) {
	std::ostringstream os; string ost;
	char *dir = new char[PATH_MAX +1];
	getdir(dir);
	/*if(subm != Perl) {*/
	if(mode != Perl) {
		os << dir << "/" << name << " " << str << std::ends;
	} else {
		os << "perl -W " << dir << "/" << full << " " << str << std::ends;
	}
	ost = os.str();
	if(msystem(ost.c_str()) < 0) {
		sys_msg("Error: exec msystem");
		std::terminate();
	}
	delete (dir);
	return (0);
}


	void copy_right(unsigned long n,string & str) {
		unsigned int x,y,*index;
			for(x=0; x<8; x++) {
			index = &x;
			*index |= 0x10;
		switch(n & 0x0f) {
			case 0:  y = str.find_first_of(0x65);
					 if((y == string::npos) || (y != 0x11))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 1:  y = str.find_last_of (0x6e);
					 if((y == string::npos) || (y != 0x12))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 2:  y = str.find_first_of(0x4c);
					 if((y == string::npos) || (y != 0x13))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 3:  y = str.find_last_of (0x78);
					 if((y == string::npos) || (y != 0xd))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 4:  y = str.find_first_of(0x4d);
					 if((y == string::npos) || (y != 0xe))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 5:  y = str.find_last_of (0x75);
					 if((y == string::npos) || (y != 0xf))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 6:  y = str.find_first_of(0x68);
					 if((y == string::npos) || (y != 0x14))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 7:  y = str.find_last_of (0x61);
					 if((y == string::npos) || (y != 0x15))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 8:  y = str.find_first_of(0x69);
					 if((y == string::npos) || (y != 0x15))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 9:  y = str.find_last_of (0x65);
					 if((y == string::npos) || (y != 0x14))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 10: y = str.find_first_of(0x45);
					 if((y == string::npos) || (y != 0xe))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 11: y = str.find_last_of (0x69);
					 if((y == string::npos) || (y != 0xf))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *indx &= ~0x10; n >>= 4;
					 break;
			case 12: y = str.find_first_of(0x67);
					 if((y == string::npos) || (y != 0x12))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 13: y = str.find_last_of (0x54);
					 if((y == string::npos) || (y != 0x11))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 14: y = str.find_first_of(0x6b);
					 if((y == string::npos) || (y != 0x10))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			case 15: y = str.find_last_of (0x72);
					 if((y == string::npos) || (y != 0x16))
					 { errno = ENODATA; sys_err("Buffer empty"); }
					 *index &= ~0x10; n >>= 4;
					 break;
			default: exit(1); break;
		}
	}
	std::cout << buffer << std::endl;
}

} // end of MJL
